#!/usr/bin/python
from __future__ import with_statement

BASEDIR = "/home/nasroot/foscripts"
PROGRAM = "is_under_execution"

import sys
import os
import fcntl
import errno

sys.path.append(os.path.join(BASEDIR, "bin/folib"))
import Utility

sys.path.append(os.path.join(BASEDIR, "bin"))
import Config
import Exceptions
from CommonLogger import log_debug, log_info, log_warning, log_error, log_exception, setup_rotation_logger
import lock_common

def main():
    # Enable Ctrl-C interruption
    Utility.enable_interrupt()

    # Load configs
    config = Config.Config()

    # Setup logging
    log_file_path = os.path.join(config.log_dir, config.common_log_file)
    setup_rotation_logger(__name__, config.log_level, log_file_path, config.log_size, config.log_gen)

    log_info("----------------------- %s --------------------------" % PROGRAM)

    ### Acquire execution lock
    # Lock on self node
    execute_lock_path = lock_common.get_execute_lock_file()
    with open(execute_lock_path, 'w') as locker:
        try:
            fcntl.flock(locker, fcntl.LOCK_EX | fcntl.LOCK_NB)
        except EnvironmentError, e:
            if e.errno == errno.EAGAIN:
                log_info("Execution lock is acquired.")
                raise Exceptions.NoExecution()
            else:
                log_error("An error occurred while getting a lock for execution. error=%s" % str(e))
                raise
        else:
            log_info("Execution lock is not acquired.")

if __name__ == '__main__':
    try:
        main()
    except Config.BaseError, e:
        print "Unable to load script configuration: %s" % str(e)
        sys.exit(10)
    except KeyboardInterrupt:
        print "Interrupted."
        log_warning("Interrupted.")
        sys.exit(2)
    except Exceptions.NoExecution:
        # Return 0 if it cannot get lock
        sys.exit(0)
    except EnvironmentError, e:
        log_exception("A system error occurred: %s", e)
        sys.exit(99)
    except Exception, e:
        log_exception("An internal error occurred: %s", e)
        sys.exit(99)

    # Return 1 if it can get lock
    sys.exit(1)
