#!/usr/bin/python
from __future__ import with_statement

BASEDIR = "/home/nasroot/foscripts"
PROGRAM = "swap_hcp_domain"

import sys
import os
import getopt

sys.path.append(os.path.join(BASEDIR, "bin/folib"))
import Utility
import HDIClient
import HDIOperation

sys.path.append(os.path.join(BASEDIR, "bin"))
import Config
import Exceptions
from CommonLogger import log_debug, log_info, log_warning, log_error, log_exception, setup_rotation_logger

def usage():
    print
    print "Usage:"
    print "    sudo maintexec %s --primary <HCP domain for primary>"
    print

def main():
    # Enable Ctrl-C interruption
    Utility.enable_interrupt()

    # Load configs
    config = Config.Config()

    # Setup logging
    log_file_path = os.path.join(config.log_dir, config.common_log_file)
    setup_rotation_logger(__name__, config.log_level, log_file_path, config.log_size, config.log_gen)

    log_info("----------------------- %s --------------------------" % PROGRAM)

    # Parse options
    short_opts = ''
    long_opts = ['primary=']
    try:
        opts, args = getopt.getopt(sys.argv[1:], short_opts, long_opts)
    except getopt.GetoptError, e:
        raise Exceptions.OptionError(e)

    primary = None
    for o, a in opts:
        if o == '--primary':
            primary = a

    if primary is None:
        raise Exceptions.OptionError("No primary HCP is specified.")
    log_info("Specified primary: %s" % primary)

    # Setup connection to localhost.
    # No credential is actually required but use this interface.
    cred = HDIClient.Credential(None, None, None, None)
    conn = HDIOperation.Connection('localhost', cred, config.retry_count, config.retry_interval, config.timeout)

    # Get the current configuration
    hcpop = HDIOperation.HCPSetting(conn)
    hcp_info = hcpop.get()

    # If the specified primary HCP is already set as primary HCP, do nothing.
    if hcp_info.domain == primary:
        log_info("The primary HCP domain is already %s. Do nothing." % primary)
        raise Exceptions.NoExecution()

    # If the specified primary HCP is not the primary HCP defined on this node,
    # it has to be defined as replica HCP on this node.
    if hcp_info.rephost != primary:
        log_error("Neither primary HCP nor replica HCP defined on this node is different from the specified primary HCP. primary=%s, replica=%s, specified=%s" % (hcp_info.domain, hcp_info.rephost, primary))
        raise Exceptions.ConfigurationError("HCP setting on this node is not an expected setting.")

    # Swap primary and replica
    try:
        hcp_info = hcpop.swap_with_replica()
    except HDIOperation.OperationError, e:
        log_error("Failed to swap primary HCP and replica HCP on this node. error=%s" % str(e))
        raise Exceptions.OperationError("Failed to swap primary HCP and replica HCP on this node.", e.code)

    if hcp_info.domain != primary:
        log_error("Primary HCP domain is not the specified domain even after swapped! primary=%s, specified=%s" % (hcp_info.domain, primary))
        raise Exceptions.OperationError("Swapping HCP domains did not have any effect.", -1)

if __name__ == '__main__':
    try:
        main()
    except Exceptions.NoExecution:
        sys.exit(0)
    except Exceptions.OptionError, e:
        log_error("Option error: %s" % str(e))
        usage()
        sys.exit(1)
    except KeyboardInterrupt:
        print "Interrupted."
        log_warning("Interrupted.")
        sys.exit(2)
    except Config.BaseError, e:
        print "Unable to load script configuration: %s" % str(e)
        sys.exit(10)
    except Exceptions.ConfigurationError, e:
        log_error("Unexpected configuration found: %s" % str(e))
        sys.exit(20)
    except Exceptions.OperationError, e:
        log_error("An operation error occurred: %s" % str(e))
        sys.exit(90)
    except EnvironmentError, e:
        log_exception("A system error occurred: %s", e)
        sys.exit(99)
    except Exception, e:
        log_exception("An internal error occurred: %s", e)
        sys.exit(99)

    sys.exit(0)
